@@ -389,22 +389,19 @@ def init_theta(h_mult, n_qubits, J_eff, degrees):
     return theta
 
 
-@njit
 def init_thresholds(n_qubits):
     n_bias = n_qubits - 1
     thresholds = np.empty(n_bias, dtype=np.float64)
     tot_prob = 0
-    p = 1.0
-    if n_qubits & 1:
-        q = n_qubits // 2
-        thresholds[q - 1] = p
-        tot_prob = p
-        p /= 2
-    for q in range(1, n_qubits // 2):
+    p = n_qubits
+    for q in range(1, n_qubits >> 1):
         thresholds[q - 1] = p
         thresholds[n_bias - q] = p
         tot_prob += 2 * p
-        p /= 2
+        p = math.comb(n_qubits, q + 1)
+    if n_qubits & 1:
+        thresholds[n_qubits >> 1] = p
+        tot_prob += p
     thresholds /= tot_prob
 
     return thresholds
@@ -442,9 +439,7 @@ def probability_by_hamming_weight(J, h, z, theta, t, n_bias, normalized=True):
 
 
 @njit(parallel=True)
-def maxcut_hamming_cdf(n_qubits, J_func, degrees, quality, tot_t, h_mult):
-    hamming_prob = init_thresholds(n_qubits)
-
+def maxcut_hamming_cdf(hamming_prob, n_qubits, J_func, degrees, quality, tot_t, h_mult):
     n_steps = 1 << quality
     delta_t = tot_t / n_steps
     n_bias = n_qubits + 1
