@@ -15,37 +15,41 @@ def get_otoc_hamming_distribution(J=-1.0, h=2.0, z=4, theta=0.0, t=5, n_qubits=6
         bias[0] = 1.0
         return bias
 
-    max_entropy = np.empty(n_bias, dtype=np.float64)
+    diff_x = np.empty(n_bias, dtype=np.float64)
     tot_prob = 0
     p = 1.0
     for q in range(n_qubits >> 1):
-        max_entropy[q] = p
-        max_entropy[n_bias - (q + 1)] = p
+        diff_x[q] = p
+        diff_x[n_bias - (q + 1)] = p
         tot_prob += 2 * p
         p = math.comb(n_qubits, q + 1)
     if n_qubits & 1:
-        max_entropy[n_qubits >> 1] = p
+        diff_x[n_qubits >> 1] = p
         tot_prob += p
-    max_entropy /= tot_prob
+    diff_x *= n_qubits / tot_prob
 
     signal_frac = 0.0
     diff_z = np.zeros(n_bias, dtype=np.float64)
+    diff_z[0] = n_qubits
     for pauli_string in pauli_strings:
         pauli_string = list(pauli_string)
         if len(pauli_string) != n_qubits:
             raise ValueError("OTOCS pauli_string must be same length as n_qubits! (Use 'I' for qubits that aren't changed.)")
 
-        signal_frac -= pauli_string.count('Y') + pauli_string.count('Z')
+        term_signal = 0.5 * pauli_string.count('X') + pauli_string.count('Z') + 1.5 * pauli_string.count('Y')
+        if term_signal == 0:
+            continue
+
+        signal_frac -= term_signal
 
         fwd = probability_by_hamming_weight(J, h, z, theta, t, n_qubits + 1)
         rev = probability_by_hamming_weight(-J, -h, z, theta + np.pi, t, n_qubits + 1)
         diff_theta = rev - fwd
-        diff_theta[0] += 1.0
 
         phi = theta + np.pi / 2
         fwd = probability_by_hamming_weight(-h, -J, z, phi, t, n_qubits + 1)
         rev = probability_by_hamming_weight(h, J, z, phi - np.pi, t, n_qubits + 1)
-        diff_phi = (rev - fwd) + max_entropy
+        diff_phi = rev - fwd
 
         diff_lam = (diff_theta + diff_phi) / 2
 
@@ -54,17 +58,19 @@ def get_otoc_hamming_distribution(J=-1.0, h=2.0, z=4, theta=0.0, t=5, n_qubits=6
                 case 'X':
                     diff_z += diff_theta
                 case 'Z':
-                    diff_z += diff_phi
+                    diff_x += diff_phi
                 case 'Y':
-                    diff_z += diff_lam
+                    diff_z += diff_theta
+                    diff_x += diff_phi
                 case _:
-                    diff_z[0] += 1.0
+                    pass
 
     # Normalize:
     diff_z /= diff_z.sum()
+    diff_x /= diff_x.sum()
 
     signal_frac = 2 ** signal_frac
-    diff_z = signal_frac * diff_z + (1 - signal_frac) * max_entropy
+    diff_z = signal_frac * diff_z + (1 - signal_frac) * diff_x
 
     # Normalize:
     diff_z /= diff_z.sum()
@@ -135,24 +141,24 @@ def take_sample(n_qubits, sample, m, inv_dist):
     return sample
 
 
-def get_willow_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len):
+def get_willow_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len, t):
     inv_dist = np.zeros(n_qubits, dtype=np.float64)
     for idx in butterfly_idx_x:
         b_row, b_col = divmod(idx, row_len)
         for q in range(n_qubits):
             q_row, q_col = divmod(q, row_len)
-            inv_dist[q] -= abs(q_row - b_row) + abs(q_col - b_col)
+            inv_dist[q] += abs(q_row - b_row) + abs(q_col - b_col)
     for idx in butterfly_idx_z:
         b_row, b_col = divmod(idx, row_len)
         for q in range(n_qubits):
             q_row, q_col = divmod(q, row_len)
-            inv_dist[q] += abs(q_row - b_row) + abs(q_col - b_col)
-    inv_dist -= inv_dist.min()
+            inv_dist[q] -= abs(q_row - b_row) + abs(q_col - b_col)
+    inv_dist = 2 ** (inv_dist / t)
 
     return inv_dist
 
 
-def get_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len):
+def get_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len, t):
     inv_dist = np.zeros(n_qubits, dtype=np.float64)
     half_row = row_len >> 1
     half_col = col_len >> 1
@@ -166,7 +172,7 @@ def get_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len):
             col_d = abs(q_col - b_col)
             if col_d > half_col:
                 col_d = col_len - col_d
-            inv_dist[q] -= row_d + col_d
+            inv_dist[q] += row_d + col_d
     for idx in butterfly_idx_z:
         b_row, b_col = divmod(idx, row_len)
         for q in range(n_qubits):
@@ -177,8 +183,8 @@ def get_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len):
             col_d = abs(q_col - b_col)
             if col_d > half_col:
                 col_d = col_len - col_d
-            inv_dist[q] += row_d + col_d
-    inv_dist -= inv_dist.min()
+            inv_dist[q] -= row_d + col_d
+    inv_dist = 2 ** (inv_dist / t)
 
     return inv_dist
 
@@ -186,21 +192,17 @@ def get_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len):
 def generate_otoc_samples(J=-1.0, h=2.0, z=4, theta=0.0, t=5, n_qubits=65, pauli_strings = ['X' + 'I' * 64], shots=100, is_orbifold=True):
     thresholds = fix_cdf(get_otoc_hamming_distribution(J, h, z, theta, t, n_qubits, pauli_strings))
 
-    signal_frac = 0.0
-    for pauli_string in pauli_strings:
-        pauli_string = list(pauli_string)
-        signal_frac -= pauli_string.count('Y') + pauli_string.count('Z')
-    signal_frac = 2 ** signal_frac
-
     row_len, col_len = factor_width(n_qubits)
     inv_dist = np.zeros(n_qubits, dtype=np.float64)
     for pauli_string in pauli_strings:
+        if (pauli_string.count('X') + pauli_string.count('Y') + pauli_string.count('Z')) == 0:
+            continue
         butterfly_idx_x = find_all_str_occurrences(pauli_string, 'X')
         butterfly_idx_z = find_all_str_occurrences(pauli_string, 'Z')
         if is_orbifold:
-            inv_dist += get_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len)
+            inv_dist += get_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len, t)
         else:
-            inv_dist += get_willow_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len)
+            inv_dist += get_willow_inv_dist(butterfly_idx_x, butterfly_idx_z, n_qubits, row_len, col_len, t)
         inv_dist /= 2.0
 
     qubit_pows = [1 << q for q in range(n_qubits)]
@@ -216,13 +218,6 @@ def generate_otoc_samples(J=-1.0, h=2.0, z=4, theta=0.0, t=5, n_qubits=65, pauli
             continue
 
         # Second dimension: permutation within Hamming weight
-        if np.random.random() < signal_frac:
-            samples.append(take_sample(n_qubits, 0, m, inv_dist))
-        else:
-            bit_pows = np.random.choice(qubit_pows, size=m, replace=False)
-            sample = 0
-            for bit_pow in bit_pows:
-                sample |= bit_pow
-            samples.append(sample)
+        samples.append(take_sample(n_qubits, 0, m, inv_dist))
 
     return samples
